
#include <ESP8266WiFi.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
#include <ezTime.h> // ntp Time Server
#include <ESPAsyncTCP.h>
#include <ESPAsyncWebServer.h>

Adafruit_BME280 bme; // I2C

#define LDRPin A0  // LDR analog input pin

int LDRValue = 0;
float voltage = 0.0; 
float t = 0.0; // temp
float h = 0.0; // humidity
float p = 0.0; // pressure
unsigned int interval = 10000; // 1000 equals 1 second
String dayNight = "";
String localTime = "";
Timezone myTZ;

// Replace with your network credentials
const char* ssid = "xxxxxxxx";// put your WiFi SSID here
const char* password = "xxxxxxxxxx"; // and your password here

// Create AsyncWebServer object on port 80
AsyncWebServer server(80);

unsigned long previousMillis = 0;    // will store last time DHT was updated
const char index_html[] PROGMEM = R"rawliteral(
<!DOCTYPE HTML><html>
<head>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.7.2/css/all.css" integrity="sha384-fnmOCqbTlWIlj8LyTjo7mOUStjsKC4pOpQbqyi7RrhN7udi9RwhKkMHpvLbHG9Sr" crossorigin="anonymous">
  <style>
    html {
     font-family: Arial;
     display: inline-block;
     margin: 0px auto;
     text-align: center;
    }
    h2 { font-size: 3.0rem; }
    p { font-size: 3.0rem; }
    .units { font-size: 1.2rem; }
    .dht-labels{
      font-size: 1.5rem;
      vertical-align:middle;
      padding-bottom: 15px;
    }
  </style>
</head>
<body>
  <h2>WiFi Weather Station</h2>
    <p>
    <span id="localTime">%LOCALTIME%</span>
  </p>  
    <p>
    <i class="fas fa-tint" style="color:#00add6;"></i> 
    <span class="dht-labels">Humidity</span>
    <span id="humidity">%HUMIDITY%</span>
    <sup class="units">%</sup>
  </p>
  <p>
    <i class="fas fa-thermometer-half" style="color:#059e8a;"></i> 
    <span class="dht-labels">Temperature</span> 
    <span id="temperature">%TEMPERATURE%</span>
    <sup class="units">&deg;C</sup>
  </p>
  <p>
    <i class="fas fa-arrow-circle-down"></i>
    <span class="dht-labels">Pressure</span> 
    <span id="pressure">%PRESSURE%</span>
    <sup class="units">hPa</sup>
  </p>
  <p>
    <i class="fas fa-globe-africa"></i> 
    <span class="dht-labels">Outside, is it light or dark?</span>
    <span id="dayNight">%DAYNIGHT%</span>
  </p>
</body>
<script>

//    <span id="localTime">%LOCALTIME%</span>

setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("temperature").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/temperature", true);
  xhttp.send();
}, 10000 ) ;

setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("dayNight").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/dayNight", true);
  xhttp.send();
}, 10000 ) ;

setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("humidity").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/humidity", true);
  xhttp.send();
}, 10000 ) ;

setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("pressure").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/pressure", true);
  xhttp.send();
}, 10000 ) ;

setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("localTime").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/localTime", true);
  xhttp.send();
}, 10000 ) ;
</script>
</html>)rawliteral";

// Replaces placeholder with DHT values
String processor(const String& var){
  Serial.println(var);
  if(var == "TEMPERATURE"){
    return String(t);
  }
  else if(var == "HUMIDITY"){
    return String(h);
  }
  else if(var == "PRESSURE"){
    return String(p);
  }
  else if(var == "DAYNIGHT"){
    return String(dayNight);
  }
  else if(var == "LOCALTIME"){
    return String(localTime);
  }
  return String();
}

void setup() {
    Serial.begin(9600);
    while(!Serial);    // time to get serial running
    bme.begin();  // start sensor BME280
      // Connect to Wi-Fi
    WiFi.hostname("BMEHost");
    WiFi.begin(ssid, password);
    Serial.println("Connecting to WiFi");
    while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.println(".");
    }

    // Print ESP8266 Local IP Address
    Serial.println(WiFi.localIP());
    delay(1000);
    Serial.println(" ");

    waitForSync(); // time server sync

  // Route for root / web page
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/html", index_html, processor);
  });
  server.on("/temperature", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", String(t).c_str());
  });
  server.on("/humidity", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", String(h).c_str());
  });

  server.on("/pressure", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", String(p).c_str());
  });

// post the day/night value to the web site
  server.on("/dayNight", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", String(dayNight).c_str());
  });

// post the localTime value to the web site
  server.on("/localTime", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", String(localTime).c_str());
  });

  // Start server
  server.begin();

} 

void loop() { 
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval) {
    // save the last time you updated the DHT values
    previousMillis = currentMillis;

    ////////////////////////////////////////////////////
/// print UTC and Local time
  Serial.println(UTC.dateTime());
  if (myTZ.setLocation("Europe/London")) {
  Serial.println(myTZ.dateTime());
  localTime = myTZ.dateTime();
  } else {
  Serial.println(errorString());
  }
  
    // Read temperature as Celsius (the default)
    float newT = bme.readTemperature();
    if (isnan(newT)) {
      Serial.println("Failed to read from temperature sensor!");
    }
    else {
      t = newT;
      Serial.println(t);
    }
    // Read Humidity
    float newH = bme.readHumidity();
    // if humidity read failed, don't change h value 
    if (isnan(newH)) {
      Serial.println("Failed to read from humidity sensor!");
    }
    else {
      h = newH;
      Serial.println(h);
    }

    // Read Pressure
    float newP = bme.readPressure();
    // if pressure read failed, don't change p value 
    if (isnan(newP)) {
      Serial.println("Failed to read from pressure sensor!");
    }
    else {
      p = newP / 100.0;
      Serial.println(p);
    }
//////////////////////////////////////////////////
/// add day/night calc here
    // read the analog pin for the LDR
    LDRValue = analogRead(LDRPin);
    voltage = LDRValue * (3.3 / 1023);
    if (voltage < 1) {
      dayNight = "Night Time";
      }
      else {dayNight = "Day Light";
      }
    Serial.println(dayNight);
///////////////////////////////////////////////

  }
}
